#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import matplotlib.pyplot as plt
import numpy as np
import csv
import sys
import os
import common
import parseMe

# Sample data


def plotBars(ax, ticklabels, summary, timeDesc=""):
    barWidth = 0.3
    dataSize = len(summary[parseMe.args.ioTypes[0]])
    br = {0: np.arange(dataSize)}
    for i in range(1, dataSize):
        br[i] = [x + barWidth for x in br[i - 1]]

    for i, which in enumerate(parseMe.args.ioTypes):
        ax.bar(br[i], summary[which], 0.3, label=which)

    ax.set_ylabel("NumCalls")
    ax.set_title("ADIOS numCalls:" + timeDesc)

    ax.set_xticks([r + barWidth for r in range(dataSize)], ticklabels)

    # Add legend
    ax.legend()


def summarizeFile(inputFile):
    try:
        with open(inputFile, "r") as f:
            data = csv.reader(f, delimiter=",", quotechar="|")
            data1 = [float(row[0]) for row in data if row]  # Handle empty rows
            return int(data1[0]) if data1 else 0  # num of calls are the same for all ranks
    except (FileNotFoundError, IndexError, ValueError) as e:
        print(f"Warning: Could not process {inputFile}: {e}")
        return 0


if __name__ == "__main__":
    print("Script name:", sys.argv[0])
    if len(sys.argv) < 2:
        print("Need Arguments for files")
        sys.exit()

    found_async = [item for item in parseMe.args.ioTypes if "async" in item.lower()]

    if found_async:
        keywords = ["PP", "PDW", "ES", "BS_WaitOnAsync", "DC_WaitOnAsync1", "DC_WaitOnAsync2"]
        ticklabels = [
            "PP",
            "PDW",
            "ES",
            "BS\nWaitOn\nAsync",
            "DC\nWaitOn\nAsync1",
            "DC\nWaitOn\nAsync2",
        ]
    else:
        keywords = ["PP", "PDW", "ES"]
        ticklabels = ["PP", "PDW", "ES"]

    summary = {key: [] for key in parseMe.args.ioTypes}
    # barLabels = list(parseMe.args.ioTypes)

    fig, ax1 = plt.subplots()

    for which in parseMe.args.ioTypes:
        for key in keywords:
            inputFile = (
                parseMe.command_options[parseMe.TAGS["input_dir"]] + which + "_nCalls_" + key
            )
            if os.path.exists(inputFile):
                summary[which].append(summarizeFile(inputFile))
            else:
                summary[which].append(0)

    if len(summary[parseMe.args.ioTypes[0]]) == 0:
        print("Nothing to plot")
        sys.exit()

    plotBars(ax1, ticklabels, summary)
    plt.legend()

    # Show plot
    figName = parseMe.command_options[parseMe.TAGS["out"]] + "_nCalls"
    common.saveMe(figName)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import numpy
import adios2

#
# Create many files for a campaign, each with a single small array
#

NFILES = 100

# User data
myArray = numpy.arange(0.0, 9.0, 1)
Nx = myArray.size

for n in range(NFILES):
    myArray += 1.0
    with adios2.Stream(f"data{n:03}.bp", "w") as fh:
        fh.write("d", myArray, [Nx], [0], [Nx])



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0


import numpy
import adios2

# User data
NRows = 5
NCols = 6

shape = [NRows, NCols]
start = [0, 0]
count = [NRows, NCols]

temperatures = numpy.zeros(NRows * NCols, dtype=numpy.int16)

value = (NRows * NCols) + 1
for i in range(0, NRows):
    for j in range(0, NCols):
        temperatures[i * NCols + j] = value
        value = value + 1

# ADIOS2 high-level API for Write
fw = adios2.open("test1.bp", "w")
fw.write("note", "This is an ADIOS2 output")
fw.write("temperature2D", temperatures, shape, start, count)
fw.write("nrows", numpy.array([NRows]))
fw.write("ncols", numpy.array([NCols]))
fw.close()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import matplotlib.pyplot as plt
import numpy as np
import csv
import sys
import os
import parseMe
import common


def drawBars(ax, labels, summary, timeDesc):
    """Draw stacked bar chart for ADIOS timing data."""
    width = 0.6
    bottom = np.zeros(len(labels))

    # Stack bars for each keyword
    for key in ["ES", "PP", "PDW"]:
        if summary[key]:  # Only plot if data exists
            ax.bar(labels, summary[key], width, bottom=bottom, label=key)
            bottom += np.array(summary[key])  # Update bottom for stacking

    # Separate bars for other metrics (not stacked)
    ax.bar(labels, summary["ES_aggregate_info"], width / 2, label="ES_aggregate_info")
    ax.bar(
        labels, summary["ES_AWD"], width / 2, bottom=summary["ES_aggregate_info"], label="ES_AWD"
    )
    ax.bar(labels, summary["FixedMetaInfoGather"], width / 3, label="FixedMetaInfoGather")

    ax.set_ylabel("Seconds")
    ax.set_title(f"ADIOS Time: {timeDesc}")
    ax.legend()


def summarizeFile(inputFile, whichRank=0):
    """Read CSV file and return value for specified rank or max."""
    data = csv.reader(open(inputFile, "r"), delimiter=",", quotechar="|")

    data1 = np.array([row[0] for row in data], dtype=float)

    if whichRank < 0:
        return np.max(data1)
    elif whichRank < len(data1):
        return data1[whichRank]
    else:
        return data1[0]


if __name__ == "__main__":
    print("Script name:", sys.argv[0])
    if len(sys.argv) < 2:
        print("Need Arguments for files")
        sys.exit()

    keywords = [
        "PP",
        "PDW",
        "ES",
        "ES_AWD",
        "ES_aggregate_info",
        "MetaInfoBcast",
        "FixedMetaInfoGather",
    ]
    summary = {}
    for key in keywords:
        summary[key] = []

    barLabels = []
    fig, ax1 = plt.subplots()

    # print (parseMe.args.ioTypes)
    whichRank = parseMe.command_options[parseMe.TAGS["rank"]]
    # for i in range(0, len(parseMe.args.ioTypes)):
    # which=sys.argv[i] ## default flatten joined
    #    which=parseMe.args.ioTypes[i]
    for which in parseMe.args.ioTypes:
        barLabels.append(which)
        for key in keywords:
            inputFile = parseMe.command_options[parseMe.TAGS["input_dir"]] + which + "_secs_" + key
            if os.path.exists(inputFile):
                summary[key].append(summarizeFile(inputFile, whichRank))
            else:
                print("No such file: " + inputFile)

    # print ("summary = ", summary)
    # print (barLabels)

    if len(summary["ES"]) == 0:
        print("Nothing to plot")
        sys.exit()

    timeDesc = "Max" if whichRank < 0 else f"Rank {whichRank}"
    drawBars(ax1, barLabels, summary, timeDesc)

    figName = parseMe.command_options[parseMe.TAGS["out"]]
    if figName.endswith("/"):
        figName += "max" if whichRank < 0 else f"rank_{whichRank}"
    else:
        figName += "_max" if whichRank < 0 else f"_rank_{whichRank}"
    figName += ".png"  # Add extension

    common.saveMe(figName)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

#
# Demonstrates writing Values (single element variables) with ADIOS
# GlobalValue:  a single value written by one process
# LocalValue:   one value per process, represented as 1D GlobalArray at reading
#   LocalValue must be defined as an adios2.Variable first and used in
#       subsequent write() calls.
#
from mpi4py import MPI
import numpy
import adios2
import random

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# User data
myArray = numpy.array([0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0])
myArray = myArray + (rank * 10)
Nx = myArray.size

NSTEPS = 5

with adios2.Stream("values-py.bp", "w", comm) as fh:
    # Either use begin_step/end_step explicitely within a loop, or
    # use stepping ADIOS stream, then use fh.current_step() for the step value
    vnparts = fh._io.define_variable("Nparts", rank, [adios2.LocalValueDim])
    vprocid = fh._io.define_variable("ProcessID", rank, [adios2.LocalValueDim])
    for _ in fh.steps(NSTEPS):
        if fh.current_step() == 0:
            # This does not work:
            #    fh.write("ProcessID", rank, [adios2.LocalValueDim], [], [])
            fh.write(vprocid, rank)
            if rank == 0:
                fh.write("Nproc", size)

        if rank == 0:
            fh.write("GlobalString", f"This is step {fh.current_step()}")
            fh.write("Step", fh.current_step())

        # random size per process, 5..10 each
        Nparts = random.randint(0, 5) + 5
        fh.write(vnparts, Nparts)

        fh.write("a", myArray, [size * Nx], [rank * Nx], [Nx])
        fh.write("ajoined", myArray, [adios2.JoinedDim], [], [Nx])


# Read content:
# bpls -l values-py.bp
# bpls -l values-py.bp -D
# bpls -l values-py.bp -d



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

from adios2 import Stream
from adios2.bindings import DerivedVarType
import unittest
import numpy as np


class TestDerivedVariable(unittest.TestCase):
    FILENAME = "pythontestderivedvariable.bp"
    EXPR = "t = temps\nadd(t, t)"
    TEMP = np.array([35, 40, 30, 45], dtype=np.int64)

    def test_01_create_write(self):
        with Stream(self.FILENAME, "w") as f:
            temps = f.io.define_variable("temps", self.TEMP, self.TEMP.shape, [0], self.TEMP.shape)
            dmo = f.io.define_derived_variable("derived/metadataonly", self.EXPR)
            ds = f.io.define_derived_variable(
                "derived/storedata", self.EXPR, DerivedVarType.StoreData
            )
            de = f.io.define_derived_variable(
                "derived/expressionstring", self.EXPR, DerivedVarType.ExpressionString
            )
            f.write("temps", self.TEMP)
            self.assertEqual(temps.name(), "temps")
            self.assertEqual(temps.block_id(), 0)
            self.assertEqual(temps.count(), [4])
            self.assertEqual(temps.shape(), [4])
            self.assertEqual(temps.sizeof(), 8)
            self.assertEqual(dmo.name(), "derived/metadataonly")
            self.assertEqual(dmo.type(), DerivedVarType.StatsOnly)
            self.assertEqual(ds.name(), "derived/storedata")
            self.assertEqual(ds.type(), DerivedVarType.StoreData)
            self.assertEqual(de.name(), "derived/expressionstring")
            self.assertEqual(de.type(), DerivedVarType.ExpressionString)

    def test_02_create_reader(self):
        with Stream(self.FILENAME, "r") as f:
            for _ in f.steps():
                temps = f.inquire_variable("temps")
                temps_ds = f.inquire_variable("derived/storedata")
                temps_dm = f.inquire_variable("derived/metadataonly")
                temps_de = f.inquire_variable("derived/expressionstring")

                self.assertEqual(temps.name(), "temps")
                self.assertEqual(temps.block_id(), 0)
                self.assertEqual(temps.count(), [4])
                self.assertEqual(temps.sizeof(), 8)
                self.assertEqual(temps.steps(), 1)
                self.assertEqual(temps.steps_start(), 0)

                self.assertEqual(temps_ds.name(), "derived/storedata")
                self.assertEqual(temps_ds.block_id(), 0)
                self.assertEqual(temps_ds.count(), [4])
                self.assertEqual(temps_ds.sizeof(), 8)
                self.assertEqual(temps_ds.steps(), 1)
                self.assertEqual(temps_ds.steps_start(), 0)

                self.assertEqual(temps_dm.name(), "derived/metadataonly")
                self.assertEqual(temps_dm.block_id(), 0)
                self.assertEqual(temps_dm.count(), [4])
                self.assertEqual(temps_dm.sizeof(), 8)
                self.assertEqual(temps_dm.steps(), 1)
                self.assertEqual(temps_dm.steps_start(), 0)

                self.assertEqual(temps_de.name(), "derived/expressionstring")
                self.assertEqual(temps_de.block_id(), 0)
                self.assertEqual(temps_de.count(), [4])
                self.assertEqual(temps_de.sizeof(), 8)
                self.assertEqual(temps_de.steps(), 1)
                self.assertEqual(temps_de.steps_start(), 0)

                t = f.read("temps", start=[0], count=temps.count())
                if not (t == self.TEMP).all():
                    raise ValueError(
                        f"ERROR: Reading 'temps' failed. Data does not match original. data = {t}"
                    )

                ts = f.read("derived/storedata", start=[0], count=temps_ds.count())
                if not (ts == 2 * self.TEMP).all():
                    raise ValueError(
                        "ERROR: Reading 'derived/storedata' failed. "
                        f"Data does not match expected values. data = {ts}"
                    )

                ts = f.read("derived/metadataonly", start=[0], count=temps_ds.count())
                if not (ts == 2 * self.TEMP).all():
                    raise ValueError(
                        "ERROR: Reading 'derived/metadataonly' failed. "
                        f"Data does not match expected values. data = {ts}"
                    )

                ts = f.read("derived/expressionstring", start=[0], count=temps_ds.count())
                if not (ts == 2 * self.TEMP).all():
                    raise ValueError(
                        "ERROR: Reading 'derived/storedata' failed. "
                        f"Data does not match expected values. data = {ts}"
                    )


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0


from mpi4py import MPI
import numpy
import pickle
import sys
import os.path
from adios2 import Stream, FileReader

if len(sys.argv) < 1:
    print(f"Usage: {sys.argv[0]} filename")
    sys.exit(1)

filename = sys.argv[1]

if not os.path.exists(filename):
    print(f"Not found {filename}")
    sys.exit(1)

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

if not rank:
    with FileReader(filename, MPI.COMM_SELF) as f:
        md = f.get_metadata()
        print(f"md type {type(md)}  len = {len(md)}  id = {id(md)}")
else:
    md = None

comm.Barrier()
md = comm.bcast(md)
print(f"rank {rank:<4}: md type {type(md)}  len = {len(md)}  id = {id(md)}")

# Process file in a sequentialized order only for pretty printing
token = numpy.zeros(1)
status = MPI.Status()
if rank > 0:
    comm.Recv(token, rank - 1, 0, status)

with FileReader(md, filename) as f:
    vars = f.available_variables()
    print(f"File info on rank {rank}:")
    print(f"  Steps in file: {f.num_steps()}:")
    print(f"  Total number of variables: {len(vars)}:")
    max_name_length = 1
    for var in vars:
        max_name_length = max(max_name_length, len(vars))
    for var in vars:
        v = f.inquire_variable(var)
        print(f"    {v.type():<8}  {var:<{max_name_length}} ", end="")
        if len(v.shape()) > 0:
            print(f"  {v.shape()}", end="")
        print()

if rank < size - 1:
    comm.Send(token, rank + 1, 0)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import numpy as np
import cupy as cp
from adios2 import FileReader
import adios2.bindings as adios2


def write_array(fileName, nSteps, gpuArray, cpuArray):
    adios = adios2.ADIOS()
    ioWriter = adios.DeclareIO("cupyWriter")
    gpuVar = ioWriter.DefineVariable(
        "gpuArray",
        np.array([0], dtype=cpuArray.dtype),
        gpuArray.shape,
        [0] * len(gpuArray.shape),
        gpuArray.shape,
    )
    # optionally the memory space can be set to GPU
    gpuVar.SetMemorySpace(adios2.MemorySpace.GPU)
    cpuVar = ioWriter.DefineVariable(
        "cpuArray", cpuArray, cpuArray.shape, [0] * len(cpuArray.shape), cpuArray.shape
    )

    # write both cpu and gpu arrays for each simulation step
    wStream = ioWriter.Open(fileName, adios2.Mode.Write)
    for step in range(nSteps):
        # write buffers
        wStream.BeginStep()
        wStream.Put(cpuVar, cpuArray)
        wStream.Put(gpuVar, gpuArray.data.ptr, adios2.Mode.Deferred)
        wStream.EndStep()
        # update buffers
        gpuArray = gpuArray * 2
        cpuArray = cpuArray + 1
    wStream.Close()
    print(
        "Write to file %s: %s data from GPU and %s data from CPU"
        % (fileName, gpuArray.shape, cpuArray.shape)
    )


def read_array(fileName, nSteps):
    adios = adios2.ADIOS()
    ioReader = adios.DeclareIO("cupyReader")
    rStream = ioReader.Open(fileName, adios2.Mode.Read)
    for step in range(nSteps):
        rStream.BeginStep()
        # prepare input buffers
        gpuVar = ioReader.InquireVariable("gpuArray")
        cpuVar = ioReader.InquireVariable("cpuArray")
        cpuBuffer = np.zeros(cpuVar.Shape(), dtype=np.float32)
        gpuShape = gpuVar.Shape(adios2.MemorySpace.GPU)
        gpuBuffer = cp.zeros(gpuShape, dtype=np.float32)
        gpuVar.SetSelection([(0, 0), gpuShape])
        # populate data
        rStream.Get(gpuVar, gpuBuffer.data.ptr, adios2.Mode.Deferred)
        rStream.Get(cpuVar, cpuBuffer)
        rStream.EndStep()
        print("Step %d: read GPU data\n %s" % (step, gpuBuffer))
        print("Step %d: read CPU data\n %s" % (step, cpuBuffer))
    rStream.Close()


if __name__ == "__main__":
    # define simulation host data
    cpuArray = np.array([[0, 1.0, 2.0], [3.0, 4.0, 5.0]], dtype=np.float32)
    # copy the data on the device
    gpuArray = cp.asarray(cpuArray)
    print("Array allocation: ", gpuArray.device)

    mempool = cp.get_default_memory_pool()
    pinned_mempool = cp.get_default_pinned_memory_pool()
    print("Bytes required to store the gpu array", gpuArray.nbytes)
    print("Bytes allocated on the device memory pool", mempool.total_bytes())
    print("Bytes used on the device memory pool", mempool.used_bytes())
    print(
        "Blocks allocated on the pinned memory pool (The allocated pinned"
        " memory is released just after the transfer is complete)",
        pinned_mempool.n_free_blocks(),
    )

    nSteps = 2
    write_array("StepsCuPyBindings.bp", nSteps, gpuArray, cpuArray)
    read_array("StepsCuPyBindings.bp", nSteps)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

"""Manage Curve25519 keypairs for use with SealedEncryptionOperator."""

import argparse
import nacl.encoding
import nacl.public

p = argparse.ArgumentParser(
    description=__doc__, epilog="Copy ONLY the public key to shared/HPC systems for writing."
)
sub = p.add_subparsers(dest="cmd", required=True)

gen = sub.add_parser("generate", help="generate a new keypair")
gen.add_argument("public_key_file")
gen.add_argument("secret_key_file")

drv = sub.add_parser("derive", help="derive public key from an existing secret key")
drv.add_argument("secret_key_file")
drv.add_argument("public_key_file")

a = p.parse_args()

if a.cmd == "generate":
    sk = nacl.public.PrivateKey.generate()
    open(a.secret_key_file, "wb").write(sk.encode())
else:
    sk = nacl.public.PrivateKey(open(a.secret_key_file, "rb").read())

pk = sk.public_key
open(a.public_key_file, "wb").write(pk.encode())

print(f"Public key: {a.public_key_file}\n  hex: {pk.encode(nacl.encoding.HexEncoder).decode()}")
if a.cmd == "generate":
    print(f"Secret key: {a.secret_key_file}\n  hex: {sk.encode(nacl.encoding.HexEncoder).decode()}")



#!/usr/bin/env python

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

from adios2NPTypes import SmallTestData
from mpi4py import MPI
import numpy as np

try:
    import cupy as cp

    ADIOS2_HAS_CUPY = True
except ImportError:
    ADIOS2_HAS_CUPY = False
try:
    import torch

    ADIOS2_HAS_TORCH = True
except ImportError:
    ADIOS2_HAS_TORCH = False

from adios2 import Stream, LocalValueDim

comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

# Test data
data = SmallTestData(rank)
nx = data.Nx

shape = [size * nx]
start = [rank * nx]
count = [nx]

# Writer
with Stream("arrays_np.bp", "w", comm=comm) as s:
    for step in s.steps(5):
        data.update(rank, step.current_step(), size)
        s.write("varNumpy", data.u16, shape, start, count)
        if ADIOS2_HAS_CUPY:
            gpuArray = cp.asarray(data.u16)
            s.write("varCupy", gpuArray, shape, start, count)
        if ADIOS2_HAS_TORCH:
            tensor = torch.tensor(data.u16)
            s.write("varTensor", tensor, shape, start, count)

comm.Barrier()

# Reader
data = SmallTestData(rank)

with Stream("arrays_np.bp", "r", comm=comm) as fr:
    # file only
    assert fr.num_steps() == 5

    for fr_step in fr.steps():
        step = fr_step.current_step()
        data.update(rank, step, size)

        step_vars = fr_step.available_variables()
        indataU16 = np.zeros(count, dtype=data.u16.dtype)
        fr_step.read_in_buffer("varNumpy", indataU16, start, count)
        if not (indataU16 == data.u16).all():
            raise ValueError("u16 numpy array read failed")

        if ADIOS2_HAS_CUPY:
            cupyU16 = cp.zeros(count, dtype=data.u16.dtype)
            fr_step.read_in_buffer("varCupy", cupyU16, start, count)
            indataU16 = np.array(cupyU16)
            if not (indataU16 == data.u16).all():
                raise ValueError("u16 cupy array read failed")

        if ADIOS2_HAS_TORCH:
            torchU16 = torch.zeros(count, dtype=torch.uint16)
            fr_step.read_in_buffer("varTensor", torchU16, start, count)
            indataU16 = torchU16.numpy()
            if not (indataU16 == data.u16).all():
                raise ValueError("u16 torch tensor read failed")



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0


import numpy as np
from adios2 import Stream, FileReader
from time import perf_counter as pc

# User data
Nx = 100
Ny = 1000
Nvars = 1000
fname = "deferred_test.bp"

count = [Nx, Ny]
start = [0, 0]
shape = [Nx, Ny]

with Stream(fname, "w") as outf:
    data = np.random.rand(Nvars, Nx, Ny)
    print(f"Write {Nvars} arrays of [{Nx}, {Ny}] shape")
    twrite_start = pc()
    for n in range(Nvars):
        outf.write(f"data_{n:0>3}", data[n : n + 1, :, :].squeeze(), shape, start, count)
    outf.write("N", [Nvars, Nx, Ny])  # will be an array in output
    outf.write("Nx", np.array(Nx))  # will be a scalar in output
    outf.write("Ny", Ny)  # will be a scalar in output
    outf.write("Nvars", Nvars)  # will be a scalar in output
twrite_end = pc()
print(f"Time of writing: {twrite_end - twrite_start:0.6f}s")

with FileReader(fname) as inpf:
    # scalar variables are read as a numpy array with 0 dimension
    in_nx = inpf.read("Nx")
    in_ny = inpf.read("Ny")
    in_nvars = inpf.read("Nvars")
    print(f"Incoming nx, ny, nvars = {in_nx}, {in_ny}, {in_nvars}")

    tread_start = pc()
    for n in range(Nvars):
        var = inpf.inquire_variable(f"data_{n:0>3}")
        if var is not None:
            var.set_selection([start, shape])
            data = inpf.read(var)
    tread_end = pc()
    print(f"Time of reading one by one (Sync mode): {tread_end - tread_start:0.6f}s")

with FileReader(fname) as inpf:
    # scalar variables are read as a numpy array with 0 dimension
    in_nx = inpf.read("Nx")
    in_ny = inpf.read("Ny")
    in_nvars = inpf.read("Nvars")
    print(f"Incoming nx, ny, nvars = {in_nx}, {in_ny}, {in_nvars}")

    in_data = np.random.rand(Nvars, Nx, Ny)

    tread_start = pc()
    for n in range(Nvars):
        var = inpf.inquire_variable(f"data_{n:0>3}")
        if var is not None:
            var.set_selection([start, shape])
            data = inpf.read_in_buffer(var, buffer=in_data[n : n + 1, :, :], defer_read=True)
    inpf.read_complete()
    tread_end = pc()
    print(f"Time of reading at once (Deferred mode): {tread_end - tread_start:0.6f}s")



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import matplotlib.pyplot as plt
import parseMe
import os


def saveMe(figName):
    parentDir = os.path.dirname(figName)

    plt.tight_layout()
    # print (figName)
    if os.path.isdir(parentDir):
        plt.savefig(figName)
    else:
        if len(parentDir) > 1:
            print("Missing directory! Can not save plot to dir:", parentDir)
        else:
            plt.savefig(figName)

    showFig = parseMe.command_options[parseMe.TAGS["showfig"]]
    if showFig:
        plt.show()



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import json
import sys
import os
import numpy as np
import matplotlib.pyplot as plt
import re

# Require at least one JSON file argument
if len(sys.argv) < 2:
    print("Usage: python3 plot_weights.py file1.json file2.json ...")
    sys.exit(1)

json_files = sys.argv[1:]

ranks = []


def getJsonKey(key):
    unit = "MB"
    if key == "dataSize":
        jsonKey = "transport_0"
    elif key == "metadataSize":
        jsonKey = "transport_1"
    else:
        if key.endswith("_mus"):
            jsonKey = key
        else:
            jsonKey = key + "_mus"
        unit = "Sec"
    return jsonKey, unit


def getValue(obj, jsonKey):
    unitSize = 1000000

    if "_mus" in jsonKey:
        w = obj.get(jsonKey)
    elif "transport" in jsonKey:
        if obj.get(jsonKey):
            w = obj.get(jsonKey).get("wbytes")
        else:
            w = 0
        unitSize = 1048576
    else:
        print("Unable to process: ", jsonKey)

    if w:
        return float(w / unitSize)
    else:
        return 0


def retrieve(data, key):
    values = []
    jsonKey, unit = getJsonKey(key)
    for obj in data:
        values.append(getValue(obj, jsonKey))
    return unit, values


def plotKey(key):
    global json_files
    global ranks

    label = key
    for filename in json_files:
        plt.figure(figsize=(10, 4))
        try:
            with open(filename, "r") as f:
                data = json.load(f)
        except Exception as e:
            print(f"Skipping {filename}: {e}")
            continue

        label_name = filename
        if len(filename) > 100:
            label_name = os.path.basename(filename)
        if isinstance(key, str):
            unit, values = retrieve(data, key)
            if len(ranks) == 0:
                ranks = list(range(0, len(values)))
            plt.ylabel(label + " (" + unit + ")")
            plt.plot(ranks, values, marker="o", linestyle="--", label=label_name)

        if isinstance(key, (list, tuple)) and all(isinstance(item, str) for item in key):
            label = ""
            val = np.array([])
            for item in key:
                unit, values = retrieve(data, item)
                if len(val) == 0:
                    val = np.array(values)
                    label = item
                    if len(ranks) == 0:
                        ranks = list(range(0, len(values)))
                else:
                    val += values
                    label += "+" + item
                plt.plot(ranks, val, marker="o", linestyle="--", label=label_name + "_" + label)
            # plt.ylabel(label + " (" + unit + ")") ## label can be tooo long
            plt.ylabel("(" + unit + ")")
        plt.xlabel("Ranks")
        plt.title(" Values Across Ranks")
        plt.xticks(ranks)
        plt.grid(True)
        plt.tight_layout()
        plt.legend()

        output_dir = "python_plots"
        os.makedirs(output_dir, exist_ok=True)
        # Save figure
        output_path = os.path.join(output_dir, label + ".png")
        plt.savefig(output_path)
        print(f"Figure saved to {output_path}")

        plt.show()


def getProfilerComponents(key, data):
    # pattern = re.compile(r"^abc_[^_]+_mus$")
    pattern = re.compile(rf"^{key}_[^_]+_mus$")

    matches = []

    for obj in data:
        if isinstance(obj, dict):
            for key in obj:
                if pattern.match(key):
                    matches.append(key[:-4])  # remove _mus
        break

    return matches


# plot data size written on each rank
plotKey("dataSize")

# plot metadata size written on each rank
plotKey("metadataSize")

# plot end step times on each rank
plotKey("ES")
# plot PDW/PP  times on each rank
plotKey(["PDW", "PP"])
# plot indepent time blocks  on each rank
plotKey(["ES", "PDW", "PP", "BS", "DC"])


####################################################
# one can replace ES with other tags of interest.  #
# we will find all next level subcomponents        #
# and plot out if there is any valid entries       #
####################################################

if len(json_files) == 1:
    print("Exploring different  ES components ")
    testFile = json_files[0]
    with open(testFile) as f:
        data = json.load(f)
    m = getProfilerComponents("ES", data)
    if len(m) > 0:
        plotKey(m)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import argparse
from os.path import basename, exists, isdir
import glob
from adios2.bp4dbg import *


def SetupArgs():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "FILE", help="Name of the input file (.bp, .bp/md.idx, " + ".bp/md.0 or .bp/data.XXX)"
    )
    # parser.add_argument("--printdata", "-p",
    #                    help="Dump data of this variable as well", default="")
    parser.add_argument("--verbose", "-v", help="More verbosity", action="count")
    parser.add_argument(
        "--no-indextable", "-x", help="Do not print index table md.idx", action="store_true"
    )
    parser.add_argument(
        "--no-metadata", "-m", help="Do not print metadata md.0", action="store_true"
    )
    parser.add_argument("--no-data", "-d", help="Do not print data data.*", action="store_true")
    args = parser.parse_args()

    # default values
    args.idxFileName = ""
    args.dumpIdx = False
    args.metadataFileName = ""
    args.dumpMetadata = False
    args.dataFileName = ""
    args.dumpData = False

    # print("Verbosity = {0}".format(args.verbose))
    return args


def CheckFileName(args):
    if not exists(args.FILE):
        print("ERROR: File " + args.FILE + " does not exist", flush=True)
        exit(1)
    if isdir(args.FILE):
        if not args.no_indextable:
            args.idxFileName = args.FILE + "/" + "md.idx"
            args.dumpIdx = True
        if not args.no_metadata:
            args.metadataFileName = args.FILE + "/" + "md.[0-9]*"
            args.dumpMetadata = True
        if not args.no_data:
            args.dataFileName = args.FILE + "/" + "data.[0-9]*"
            args.dumpData = True
        return

    name = basename(args.FILE)
    if name.startswith("data."):
        args.dataFileName = args.FILE
        args.dumpData = True

    elif name == "md.idx":
        args.idxFileName = args.FILE
        args.dumpIdx = True

    elif name.startswith("md."):
        args.metadataFileName = args.FILE
        args.dumpMetadata = True


def DumpIndexTableFile(args):
    indexFileList = glob.glob(args.idxFileName)
    if len(indexFileList) > 0:
        DumpIndexTable(indexFileList[0])
    else:
        print("There is  no BP4 Index Table file as " + args.idxFileName)


def DumpMetadataFiles(args):
    mdFileList = glob.glob(args.metadataFileName)
    if len(mdFileList) > 0:
        for fname in mdFileList:
            DumpMetaData(fname)
    else:
        print("There are no BP4 Metadata files in   " + args.metadataFileName)


def DumpDataFiles(args):
    dataFileList = glob.glob(args.dataFileName)
    if len(dataFileList) > 0:
        for fname in dataFileList:
            DumpData(fname)
    else:
        print("There are no BP4 Data files in       " + args.dataFileName)


if __name__ == "__main__":
    args = SetupArgs()
    CheckFileName(args)
    # print(args)

    if args.dumpIdx:
        DumpIndexTableFile(args)

    if args.dumpMetadata:
        DumpMetadataFiles(args)

    if args.dumpData:
        DumpDataFiles(args)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

#
# Demonstrates writing joined array with ADIOS.
# Joined Array is a Global Array, where in one dimension we don't need to
# calculate starting offsets for each processes, nor the global dimension.
# Instead, the per-process blocks will be virtually joined together in the
# specified dimension, in the order of the blocks in the output file.
# User should not care of the order of blocks, but it will always be consistent
# ordering across multiple joined array variables as long as every process
# always writes the same number of blocks into each variable.
#
from mpi4py import MPI
import numpy
import adios2
import random

# MPI
comm = MPI.COMM_WORLD
rank = comm.Get_rank()
size = comm.Get_size()

NSTEPS = 5
NCOLS = 4

with adios2.Stream("joinedArray-py.bp", "w", comm) as fh:
    # Either use begin_step/end_step explicitely within a loop, or
    # use stepping ADIOS stream, then use fh.current_step() for the step value
    vtable = fh._io.define_variable(
        "table", numpy.zeros([1], dtype=numpy.double), [adios2.JoinedDim, NCOLS], [], [1, NCOLS]
    )
    for _ in fh.steps(NSTEPS):
        # random size per process, 5..10 each
        nrows = random.randint(0, 5) + 5
        mytable = numpy.zeros([nrows, NCOLS], dtype=numpy.double)
        for row in range(0, nrows):
            for col in range(0, NCOLS):
                mytable[row, col] = rank * 1.0 + row * 0.1 + col * 0.01

        vtable.set_selection([[], [nrows, NCOLS]])
        fh.write("table", mytable, [adios2.JoinedDim, NCOLS], [], [nrows, NCOLS])


# Read content:
# bpls -l joinedArray-py.bp/ -D -t
# bpls -l joinedArray-py.bp/ -Dd -n 4 -f "%5.2f"



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0


import adios2

# ADIOS2 high-level API for Reading
fr = adios2.open("test1.bp", "r")

inNRows = fr.read("nrows")
print("# of rows = {0}".format(inNRows[0]))

inNCols = fr.read("ncols")
print("# of cols = {0}".format(inNCols[0]))

inNote = fr.readstring("note")
print("Note = {0}".format(inNote))

inTemperatures = fr.read("temperature2D")
print("temperature2d array size = " + str(inTemperatures.size))

for row in inTemperatures:
    print("".join(["{:7}".format(item) for item in row]))

fr.close()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import matplotlib.pyplot as plt
import numpy as np
import csv
import sys
import os
import parseMe
import common


def processFile(inputFile, ax, color, tag):
    data = csv.reader(open(inputFile, "r"), delimiter=",", quotechar="|")

    data1 = np.array([row[0] for row in data], dtype=float)

    idx = np.argmax(data1 < parseMe.command_options[parseMe.TAGS["zero"]])

    if idx == 0:
        idx = data1.size

    if idx > 10:
        ax.plot(data1[0:idx], "--o", color=color)
        ax.set_xticks([0, data1.size - 1])
        # ax.set_xticks([]) #  no x ticks
    else:
        if color == "r":
            ax.plot(range(idx), data1[0:idx], "--o", color=color)
        else:
            ax.plot(range(idx), data1[0:idx], "-x", color=color)
        ax.set_xticks(range(0, idx))
        # print (data1[0:idx])
    ax.set_ylabel(tag, c=color)
    ax.set_xlabel("ranks")
    ax.tick_params(axis="y", labelcolor=color)
    ax.grid()

    if len(logScaleAxis) == 0:
        ax.ticklabel_format(useOffset=False)

    if "x" in logScaleAxis:
        ax.set_xscale("log")

    if "y" in logScaleAxis:
        ax.set_yscale("log")


if __name__ == "__main__":
    print("Script name:", sys.argv[0])
    if len(sys.argv) == 1:
        print("Need Arguments for files")
        sys.exit()

    fig, ax1 = plt.subplots()
    if len(parseMe.args.ioTypes) > 2:
        print("Unable to do 3 way comparison")
        sys.exit()
    elif len(parseMe.args.ioTypes) == 2:
        ax2 = ax1.twinx()

    # note: logscale and ticklabel_format cannot be present together. python error
    logScaleAxis = ""
    if "logScale" in parseMe.command_options:
        logScaleAxis = parseMe.command_options["logScale"]

    jsonAttrStr = parseMe.command_options[parseMe.TAGS["attr"]]
    #  'secs' or  'nCalls' or 'MB'
    whichKind = "secs"
    if "whichKind" in parseMe.command_options:
        whichKind = "MB"
    for i in range(0, len(parseMe.args.ioTypes)):
        ioType = parseMe.args.ioTypes[i]

        inputFile = (
            parseMe.command_options[parseMe.TAGS["input_dir"]]
            + ioType
            + "_"
            + whichKind
            + "_"
            + jsonAttrStr
        )
        currLabel = ioType + " " + jsonAttrStr + " ( " + whichKind + " )"
        if i == 0:
            processFile(inputFile, ax1, "r", currLabel)
        else:
            processFile(inputFile, ax2, "gray", currLabel)

    if len(parseMe.args.ioTypes) > 1:
        levelUp = False

        bottom_ax1, top_ax1 = ax1.get_ylim()
        bottom_ax2, top_ax2 = ax2.get_ylim()

        levelUp = parseMe.command_options[parseMe.TAGS["level"]]
        # else:
        #    if ( max(top_ax1, top_ax2)/min(top_ax1, top_ax2)  < 2):
        #        levelUp = true
        if levelUp:
            ax1.set_ylim(min(bottom_ax1, bottom_ax2), max(top_ax1, top_ax2))
            ax2.set_ylim(min(bottom_ax1, bottom_ax2), max(top_ax1, top_ax2))

    # plt.title('Bar Plot of Two Columns')
    # plt.legend()
    # plt.tight_layout()

    figName = parseMe.command_options[parseMe.TAGS["out"]] + "_" + jsonAttrStr
    figName = figName.replace(".", "_")
    if len(logScaleAxis) > 0:
        figName = figName + "_log_" + logScaleAxis

    common.saveMe(figName)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0


import unittest
import numpy as np
from adios2 import Adios
import adios2.bindings as bindings


class TestBPWriterStatsOnly(unittest.TestCase):
    def test_exception_at_read(self):
        # User data
        Nx = 10
        Ny = 10

        count = [Nx, Ny]
        start = [0, 0]
        shape = [Nx, Ny]

        temperatures = np.zeros(count, dtype=np.int32)

        for i in range(0, Nx):
            for j in range(0, Ny):
                temperatures[i, j] = i * Nx + j

        adios = Adios()
        with adios.declare_io("ioWriter") as ioWrite:
            varTemperature = ioWrite.define_variable(
                name="temperature2D", content=temperatures, start=start, shape=shape, count=count
            )
            varTemperature.store_stats_only(True)

            with ioWrite.open("TestWriteStatsOnly_py.bp", bindings.Mode.Write) as wStream:
                wStream.put(varTemperature, temperatures)

        with adios.declare_io("ioReader") as ioRead:
            with ioRead.open("TestWriteStatsOnly_py.bp", bindings.Mode.ReadRandomAccess) as rStream:
                var_inTemperature = ioRead.inquire_variable("temperature2D")
                assert var_inTemperature is not None
                inTemperatures = np.zeros(Nx * Ny, dtype=np.int32)
                with self.assertRaises(RuntimeError):
                    rStream.get(var_inTemperature, inTemperatures, bindings.Mode.Sync)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import argparse
import textwrap
import os

parser = argparse.ArgumentParser(description="ADIOS json file parser inputs")

TAGS = {
    "input_dir": "dataDir",  # key name for input direcory is "dataDir"
    "rank": "whichRank",  # which rank to peek profile
    "out": "plotPrefix",  # for output file name
    "showfig": "showFig",  # call plt.show or not
    "attr": "jsonAttr",  # key refer to specific json attr
    # the "zeroif" key refers to a min number to be treated as 0.
    # will process data until first rank <= this value is spotted
    "zero": "zeroIf",
    "level": "levelAxis",
}

TAGS_DEFAULT = {
    "input_dir": "outs/",  # key name for input direcory defaults to "outs/"
    "rank": 0,  # by default observe rank 0 activites for plotStack.py
    "out": "plot",  # output dir
    "showfig": "False",  # do not call plt.show by default
    "attr": "ES",  # default json attr to look at is ES
    "zero": "0.000001",  # ignore if smaller than
    "level": "False",  # whether left/right axises (if both exist) should display same range
}

parser.add_argument(
    "ioTypes",
    nargs="+",
    help="A list of known adios types in the output directory.\n"
    "Will be accessed as [" + TAGS["input_dir"] + "]/[ioType]_<secs/nCalls>_[profileTag]"
    "e.g. outs/flatten_nCalls_ES",
)

helpWithDefaults = ""
for k in TAGS:
    helpWithDefaults += TAGS[k] + "(default: " + str(TAGS_DEFAULT[k]) + ")  "

parser.add_argument(
    "--set", metavar="key=value", nargs="+", help="Accepted options:\n" + helpWithDefaults
)
args = parser.parse_args()

# print (args.set)
# print (args.file_key)


def parse_var(s):
    """
    Parse a key, value pair, separated by '=' into a tuple
    """
    items = s.split("=")
    key = items[0].strip()  # we remove blanks around keys, as is logical
    if len(items) > 1:
        # rejoin the rest:
        value = "=".join(items[1:])
    return (key, value)


def parse_vars(items):
    """
    Parse a series of key-value pairs and return a dictionary
    """
    d = {}

    if items:
        for item in items:
            key, value = parse_var(item)
            d[key] = value
    return d


# parse the key-value pairs

command_options = parse_vars(args.set)

for k in TAGS:
    if TAGS[k] not in command_options:
        command_options[TAGS[k]] = TAGS_DEFAULT[k]

if not command_options[TAGS["input_dir"]].endswith("/"):
    command_options[TAGS["input_dir"]] += "/"
if not os.path.isdir(command_options[TAGS["input_dir"]]):
    parser.error(f"Input directory '{command_options[TAGS['input_dir']]}' does not exist")

# command_options[TAGS["rank"]] = int(command_options[TAGS["rank"]])
# command_options[TAGS["zero"]] = float(command_options[TAGS["zero"]])
try:
    command_options[TAGS["rank"]] = int(command_options[TAGS["rank"]])
    if command_options[TAGS["rank"]] < 0:
        parser.error(f"Rank ({command_options[TAGS['rank']]}) must be non-negative")
except ValueError:
    parser.error(f"Rank must be an integer, got '{command_options[TAGS['rank']]}'")

try:
    command_options[TAGS["zero"]] = float(command_options[TAGS["zero"]])
    if command_options[TAGS["zero"]] < 0:
        parser.error(f"Zero threshold ({command_options[TAGS['zero']]}) must be non-negative")
except ValueError:
    parser.error(f"Zero threshold must be a number, got '{command_options[TAGS['zero']]}'")


# Safely convert levelAxis to boolean
level_val = command_options[TAGS["level"]].lower()
if level_val not in ("true", "false"):
    parser.error(f"Level must be 'True' or 'False', got '{command_options[TAGS['level']]}'")
command_options[TAGS["level"]] = level_val == "true"


showFig_val = command_options[TAGS["showfig"]].lower()
if showFig_val not in ("true", "false"):
    parser.error(f"showFig must be 'True' or 'False', got '{command_options[TAGS['level']]}'")
command_options[TAGS["showfig"]] = showFig_val == "true"

# print (command_options)



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

from adios2 import Stream, Adios
from adios2.bindings import DerivedVarType, Linf_norm, L2_norm
import unittest
import numpy as np


class TestVariable(unittest.TestCase):
    FILENAME = "pythontestvariable.bp"
    TEMP = np.array([35, 40, 30, 45], dtype=np.int64)

    def test_01_create_write(self):
        with Stream(self.FILENAME, "w") as f:
            temps = f.io.define_variable("temps", self.TEMP, self.TEMP.shape, [0], self.TEMP.shape)
            f.write(temps, self.TEMP)
            self.assertEqual(temps.name(), "temps")
            self.assertEqual(temps.block_id(), 0)
            self.assertEqual(temps.count(), [4])
            self.assertEqual(temps.shape(), [4])
            self.assertEqual(temps.sizeof(), 8)

    def test_02_create_reader(self):
        with Stream(self.FILENAME, "r") as f:
            for _ in f.steps():
                temps = f.inquire_variable("temps")
                self.assertEqual(temps.name(), "temps")
                self.assertEqual(temps.block_id(), 0)
                self.assertEqual(temps.count(), [4])
                self.assertEqual(temps.sizeof(), 8)
                self.assertEqual(temps.steps(), 1)
                self.assertEqual(temps.steps_start(), 0)

    def test_03_operators(self):
        adios = Adios()
        op1 = adios.define_operator("noop", "null")
        iow = adios.declare_io("BPWriter")

        with Stream(iow, "pythontestvariableoperators.bp", "w") as f:
            temps = f.io.define_variable("temps", self.TEMP, self.TEMP.shape, [0], self.TEMP.shape)
            temps.add_operation(op1)
            f.write(temps, self.TEMP)

        op2 = adios.define_operator("noop2", "null")
        ior = adios.declare_io("BPReader")
        with Stream(ior, "pythontestvariableoperators.bp", "r") as f:
            for _ in f.steps():
                temps = f.inquire_variable("temps")
                temps.add_operation(op2)

    def test_04_accuracy(self):
        with Stream("pythontestvariableaccuracy.bp", "w") as f:
            temps = f.io.define_variable("temps", self.TEMP, self.TEMP.shape, [0], self.TEMP.shape)
            temps.set_accuracy(0.1, Linf_norm, True)
            acc = temps.get_accuracy()
            self.assertAlmostEqual(acc.error, 0.0, 10)
            self.assertAlmostEqual(acc.norm, Linf_norm, 10)
            self.assertEqual(acc.relative, True)
            f.write(temps, self.TEMP)

        with Stream("pythontestvariableoperators.bp", "r") as f:
            for _ in f.steps():
                temps = f.inquire_variable("temps")
                temps.set_accuracy(0.1, Linf_norm, True)
                data = f.read(temps)
                self.assertEqual(data[0], self.TEMP[0])
                acc = temps.get_accuracy()
                # accuracy must be the default accuracy set in VariableBase
                # since we have no accuracy reading at the moment
                self.assertAlmostEqual(acc.error, L2_norm, 10)
                self.assertAlmostEqual(acc.norm, 0.0, 10)
                self.assertEqual(acc.relative, False)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

from adios2 import Stream, LocalValueDim
from random import randint
import numpy as np

import unittest


class TestStream(unittest.TestCase):
    def test_basic(self):
        print("===========   test_basic ==================")
        with Stream("pythonstreamtest.bp", "w") as s:
            for _ in s.steps(10):
                # Single value string
                s.write("Outlook", "Good")
                # Global array
                s.write(
                    "temp",
                    content=[randint(15, 35), randint(15, 35), randint(15, 35)],
                    shape=[3, 1],
                    start=[0, 0],
                    count=[3, 1],
                )
                # Local Value
                s.write("Wind", [5], shape=[LocalValueDim])
                # Local Array
                s.write("Coords", [38, -46], [], [], [2])
                s.write("humidity", np.random.rand(3, 1))

        with Stream("pythonstreamtest.bp", "r") as s:
            for _ in s.steps():
                minv, maxv = s.minmax("temp")
                print(f"Min/Max: var:temp step {s.current_step()} min/max = {minv}/{maxv}")
                self.assertGreaterEqual(minv, 15)
                self.assertLessEqual(maxv, 35)

                for var_name in s.available_variables():
                    print(f"var:{var_name}\t{s.read(var_name)}")
                    minv, maxv = s.minmax(var_name)
                    print(
                        f"Min/Max: var:{var_name} step {s.current_step()} min/max = {minv}/{maxv}"
                    )
                self.assertEqual(s.read("Wind", block_id=0), 5)
                self.assertEqual(s.read("Coords", block_id=0)[0], 38)
                self.assertEqual(s.read("Coords", block_id=0)[1], -46)
                self.assertEqual(s.read("humidity", block_id=0).ndim, 2)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

import numpy as np
import cupy as cp
from adios2 import Stream


def write_array(fileName, nSteps, gpuArray, cpuArray):
    with Stream(fileName, "w") as wStream:
        for _ in wStream.steps(nSteps):
            wStream.write(
                "cpuArray", cpuArray, cpuArray.shape, [0] * len(cpuArray.shape), cpuArray.shape
            )
            wStream.write(
                "gpuArray", gpuArray, gpuArray.shape, [0] * len(gpuArray.shape), gpuArray.shape
            )
            # update buffers
            gpuArray = gpuArray * 2
            cpuArray = cpuArray + 1
    print(
        "Write to file %s: %s data from GPU and %s data from CPU"
        % (fileName, gpuArray.shape, cpuArray.shape)
    )


def read_array(fileName, readGpuShape, readCpuShape):
    with Stream(fileName, "r") as rStream:
        for _ in rStream.steps():
            step = rStream.current_step()
            cpuBuffer = np.zeros(readCpuShape, dtype=np.float32)
            rStream.read("cpuArray", cpuBuffer)

            gpuBuffer = cp.zeros(readGpuShape, dtype=np.float32)
            rStream.read("gpuArray", buffer=gpuBuffer)

            print("Step %d: read GPU data\n %s" % (step, gpuBuffer))
            print("Step %d: read CPU data\n %s" % (step, cpuBuffer))


if __name__ == "__main__":
    cpuArray = np.array([[0, 1.0, 2.0], [3.0, 4.0, 5.0]], dtype=np.float32)
    gpuArray = cp.array([[0, 1.0, 2.0], [3.0, 4.0, 5.0]], dtype=np.float32)
    print("Array allocation: ", gpuArray.device)
    print("Bytes required to store the gpu array", gpuArray.nbytes)

    nSteps = 2
    write_array("StepsWriteReadTorch.bp", nSteps, gpuArray, cpuArray)
    read_array("StepsWriteReadTorch.bp", gpuArray.shape, cpuArray.shape)



#!/bin/env python3

# SPDX-FileCopyrightText: 2026 Oak Ridge National Laboratory and Contributors
#
# SPDX-License-Identifier: Apache-2.0

# Demonstrating deferred read
# multiple f.read(...) can be scheduled and executed once by
# f.read_complete()
# which allows ADIOS to use more threads to read more data concurrently.

# Run this after running the example
# mpirun -n 4 ./bin/adios2_basics_globalArrayNDWrite_mpi

import adios2
import numpy

with adios2.FileReader("globalArray.bp") as f:
    v = f.inquire_variable("GlobalArray")
    print(f"Variable GlobalArray shape = {v.shape()}, steps = {v.steps()}")
    data = f.read(v, defer_read=True)
    print(
        f"WRONG before data is read: data size = {data.size} min = {data.min()}, max = {data.max()}"
    )
    f.read_complete()
    print(
        f"CORRECT after data is read: data size = {data.size} "
        f"min = {data.min()}, max = {data.max()}"
    )
